{% if is_type_alias %}
{% if description %}
{{ description | go_comment }}
{% endif -%}
type {{ name }} {{ type_alias_value }}
{% else %}
{% if description %}
{{ description | go_comment }}
{% endif -%}
type {{ name }} struct {
{% if embedded_structs %}
	// Embedded types for inheritance
{% for embed in embedded_structs %}
	{{ embed }} `json:",inline"`
{% endfor %}
{% endif -%}
{% if fields %}
{% for field in fields.values() %}
{{ field }}
{% endfor -%}
{% endif %}
}
{% endif %}
