{#-

  Jinja2 Template for a Java class with Lombok @Data annotation
  Annotation details at https://projectlombok.org
-#}
package {{ doc.package }};

{% if metamodel_version -%}
/* metamodel_version: {{metamodel_version}} */
{% endif -%}
{% if model_version -%}
/* version: {{model_version}} */
{% endif -%}

import java.util.List;
import lombok.*;

{% if cls.source_class.description -%}
/**
  {{ cls.source_class.description }}
**/
{% endif -%}
@Data
@EqualsAndHashCode(callSuper=false)
public{% if cls.abstract %} abstract{% endif %} class {{ cls.name }} {% if cls.is_a -%} extends {{ cls.is_a }} {%- endif %} {
{%   for f in cls.fields %}
  private {{f.range}} {{ f.name }};
{%-  endfor %}

}
