// Code generated by linkml-golanggen. DO NOT EDIT.
package {{ package_name }}
{% if imports and imports.strip() %}

{{ imports }}
{% endif %}
{% if enums %}
{% for e in enums.values() %}
{{ e }}
{% endfor %}
{% endif %}
{% for s in structs.values() %}
{{ s }}
{% endfor %}
